@@ -1,6 +1,7 @@
 package DBIx::Skinny::DBD::mysql;
 use strict;
 use warnings;
+use DBIx::Skinny::SQL;
 
 sub last_insert_id {
     $_[1]->{mysql_insertid} || $_[1]->{insertid}
@@ -10,6 +11,9 @@ sub sql_for_unixtime {
     return "UNIX_TIMESTAMP()";
 }
 
+sub quote    { '`' }
+sub name_sep { '.' }
+
 sub bulk_insert {
     my ($skinny, $table, $args) = @_;
 
@@ -43,5 +47,7 @@ sub bulk_insert {
     return 1;
 }
 
+sub query_builder_class { 'DBIx::Skinny::SQL' }
+
 1;
 
